//start_config
#include <xcore/port.h>
#include <xcore/port_protocol.h>
#include <xcore/clock.h>
#include <xs1.h>

port_t RX_D   = XS1_PORT_4A;
port_t RX_DV  = XS1_PORT_1I;
port_t RX_CLK = XS1_PORT_1H;
static xclock_t clk  = XS1_CLKBLK_2;

void miiConfigReceive() {
    port_start_buffered(RX_D, 32);
    port_enable(RX_DV);
    port_enable(RX_CLK);
    clock_enable(clk);

    clock_set_source_port(clk, RX_CLK);
    port_protocol_in_strobed_slave(RX_D, RX_DV, clk);
    clock_start(clk);
}
//end_config

//start_rx_frame
#include <xcore/select.h>

void miiReceiveFrame(char *pkt) {
    int data, tail;
    int *o = ((int*)pkt)+1;
    int bytes = 0;
    
    // Wait for start of frame
    port_set_trigger_in_equal(RX_D, 0xD);
    (void) port_in(RX_D);
    port_set_trigger_in_equal(RX_DV, 0);
    
    // Receive frame data/crc
    SELECT_RES(
        CASE_THEN(RX_DV, last_data),
        CASE_THEN(RX_D, data_arrived))
    {
    data_arrived:
        // input next 32 bits of data
        data = port_in(RX_D);
        *o++ = data;
        bytes += 4;
        SELECT_CONTINUE_NO_RESET;
    last_data:
        (void) port_in(RX_DV);
        // Input any bits remaining in port
        tail = port_endin(RX_D);
        bytes += tail>>3;
        if (tail > 32) {
            tail -= 32;
            data = port_in(RX_D);
            *o++ = data;
        }
        data = port_in(RX_D);
        *o++ = data >> (32-tail);
        *(int *)pkt = bytes;
        return;
    }
}
//end_rx_frame
